local M = {}

local FSKY = scenetree.sunsky
local settingsApplied = false

-- Ttweaks version 0.1.3

-- obligatory thank you to kryotex for creating the mod from which this is trying to recreate (beamng.com/threads/96223)
-- probably wouldn't have started if I didn't have that mod as a reference. 

-- Variables --
-- I've tried to make this as user-friendly as possible for adding/changing values. Changing is self explanitory, but 
-- if you want to add a new variable you just need to either add it to the FSKYVariables table, or the otherSettings table. 
-- Followed by a corresponding entry to the settingsPreset table.
local FSKYVariables = { -- this table is for sunsky variables
    "skyBrightness",
    "shadowDistance",
    "shadowSoftness",
    "brightness",
    "exposure",
    "colorizeAmount",
    "logWeight",
    "texSize",
    "rayleighScattering",
    "flareScale",
    -- Add more variables here (sunsky only)
}

local otherSettings = { -- this table is for non sunsky variables, make sure to use the whole scenetree path.
    -- Add more variables here (with path in scenetree)
}

local defaultSettings = {}

local settingsPreset = { -- this is where you set the values for the variables
    skyBrightness = 35,
    shadowDistance = 2000,
    shadowSoftness = 0.125,
    brightness = 4,
    exposure = 0.55,
    logWeight = 0.990,
    texSize = 2048,
    rayleighScattering = 0.003,
    flareScale = 10.0,
}

-- Main Functions --

local function saveDefaultSettings()
        for _, var in ipairs(FSKYVariables) do
            defaultSettings[var] = FSKY[var]
        end
        for var, path in pairs(otherSettings) do
            defaultSettings[var] = scenetree[path]
        end
end

local function applySettings()
        for _, var in ipairs(FSKYVariables) do
            FSKY[var] = settingsPreset[var]
        end
        for var, path in pairs(otherSettings) do
            scenetree[path] = settingsPreset[var]
        end
end

local function revertSettings()
        settingsApplied = false
        for _, var in ipairs(FSKYVariables) do
            FSKY[var] = defaultSettings[var]
        end
        for var, path in pairs(otherSettings) do
            scenetree[path] = defaultSettings[var]
        end
end

local function ToggleEffect()
    if FSKY then
        local savedTOD = core_environment.getTimeOfDay()
        if settingsApplied then
            revertSettings()
            settingsApplied = false
        else
            saveDefaultSettings()
            applySettings()
            settingsApplied = true
        end
        core_environment.setTimeOfDay({ time = savedTOD.time })
    end
end

M.toggle = ToggleEffect
M.OnExtensionUnloaded = function()
    settingsApplied = true
    ToggleEffect()
end
M.onModDeactivated = function(mod)
    if mod.name == teons_tweaks then
        settingsApplied = true
        ToggleEffect()
    end
end
M.onUpdate = function() end

return M


--[[ todo list

high priority:
- force settings update function
- time of day based brightness/exposure (use TOD as a multiplier to control brightness/exposure) (TOD between two values)

low priority:
- add a GUI for changing settings
- import/export settings to a file
]]--